import unittest

import dont_argue
supply_args = dont_argue.supply_args

class TestBasicUse(unittest.TestCase):
    def test_passes_arguments(self):
        dont_argue.test_args = ['one', 'two']
        @supply_args
        def main(one, two):
            self.assertEqual(one, 'one')
            self.assertEqual(two, 'two')
        main()

    def test_passes_arguments_with_splat(self):
        dont_argue.test_args = ['one', 'two', '3', '4']
        @supply_args
        def main(one, two, *args):
            self.assertEqual(one, 'one')
            self.assertEqual(two, 'two')
            self.assertTupleEqual(args, ('3', '4'))
        main()

    def test_passes_arguments_with_splat_and_named_args(self):
        dont_argue.test_args = ['a', 'b', 'c', 'd']
        @supply_args
        def main(a, b, *args):
            self.assertEqual(a, 'a')
            self.assertEqual(b, 'b')
            self.assertTupleEqual(args, ('c', 'd'))
        main()

    def test_keyword_args(self):
        dont_argue.test_args = ['--first', 'val', '--second=blah', 'a', 'b']
        @supply_args
        def main(a, b, first=None, second=None, third=42):
            self.assertEqual(a, 'a')
            self.assertEqual(b, 'b')
            self.assertEqual(first, 'val')
            self.assertEqual(second, 'blah')
            self.assertEqual(third, 42)
        main()

    def test_splat_args_with_keywords(self):
        dont_argue.test_args = ['1', '--two', '2', '3', '4', '5']
        @supply_args
        def main(one, two=42, *args):
            self.assertEqual(one, '1')
            self.assertEqual(two, '2')
            self.assertTupleEqual(args, ('3', '4', '5'))
        main()



import argparse
from inspect import getargspec
from functools import partial

test_args = None

def supply_args(func):
    # Get argnames through introspection
    args, _, _, defaults = getargspec(func)
    parser = argparse.ArgumentParser()
    # Set up empty defaults rather than None
    defaults = defaults or []
    # Keywords match up with the last len(defaults) args
    keyword_names = args[-len(defaults):] if defaults else []
    # use the defaults with the keywords and pack them up
    keywords = {k:v for k,v in zip(keyword_names, defaults)}
    # Use what's left over as regular arguments
    args = args[:-len(keyword_names)] if keywords else args

    # Add basic arguments to argparse
    for arg in args:
        parser.add_argument(arg)

    # add keyword arguments to argparse
    for keyword, value in keywords.iteritems():
        parser.add_argument('--' + keyword, default=value)

    # Extras are unexpected arguments, they'll get splatted out.
    kwargs, extras = parser.parse_known_args(test_args)
    # Convert from 'Namespace' object
    kwargs = vars(kwargs)

    # Get all args as a list to avoid weird keyword/splat conflicts
    combined_args = [kwargs[k] for k in args + keyword_names] + extras
    # Apply the args and return the function.
    return partial(func, *combined_args)



from distutils.core import setup

with open('README.md') as f:
    long_description = f.read()

version = '0.1.3'

setup(
    name = 'dont_argue',
    packages = ['dont_argue'],
    package_data={ 'dont_argue': ['README.md'], },
    version = version,
    description = 'Dead-simple command line argument parsing',
    long_description=long_description,
    author = 'Chris Penner',
    author_email = 'christopher.penner@gmail.com',
    url = 'https://github.com/ChrisPenner/dont-argue',
    download_url = 'https://github.com/ChrisPenner/dont-argue/releases/tag/v{}'.format(version), 
    license = 'MIT',
    keywords = ['command line', 'argument', 'parsing', 'argparse'],
    classifiers = [
        'Development Status :: 3 - Alpha',

        'Topic :: Software Development :: Libraries',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
)
